#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_single_lan
short_description: Resource module for networks _appliance _single _lan
description:
  - Manage operation update of the resource networks _appliance _single _lan.
  - Update single LAN configuration.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  applianceIp:
    description: The appliance IP address of the single LAN.
    type: str
  ipv6:
    description: IPv6 configuration on the VLAN.
    suboptions:
      enabled:
        description: Enable IPv6 on VLAN.
        type: bool
      prefixAssignments:
        description: Prefix assignments on the VLAN.
        elements: dict
        suboptions:
          autonomous:
            description: Auto assign a /64 prefix from the origin to the VLAN.
            type: bool
          disabled:
            description: Disable the prefix assignment.
            type: bool
          origin:
            description: The origin of the prefix.
            suboptions:
              interfaces:
                description: Interfaces associated with the prefix.
                elements: str
                type: list
              type:
                description: Type of the origin.
                type: str
            type: dict
          staticApplianceIp6:
            description: Manual configuration of the IPv6 Appliance IP.
            type: str
          staticPrefix:
            description: Manual configuration of a /64 prefix on the VLAN.
            type: str
        type: list
    type: dict
  mandatoryDhcp:
    description: Mandatory DHCP will enforce that clients connecting to this LAN must
      use the IP address assigned by the DHCP server. Clients who use a static IP
      address won't be able to associate. Only available on firmware versions 17.0
      and above.
    suboptions:
      enabled:
        description: Enable Mandatory DHCP on LAN.
        type: bool
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  subnet:
    description: The subnet of the single LAN configuration.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance updateNetworkApplianceSingleLan
    description: Complete reference of the updateNetworkApplianceSingleLan API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-single-lan
notes:
  - SDK Method used are
    appliance.Appliance.update_network_appliance_single_lan,
  - Paths used are
    put /networks/{networkId}/appliance/singleLan,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_appliance_single_lan:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    applianceIp: string
    ipv6:
      enabled: true
      prefixAssignments:
        - autonomous: true
          disabled: true
          origin:
            interfaces:
              - string
            type: string
          staticApplianceIp6: string
          staticPrefix: string
    mandatoryDhcp:
      enabled: true
    networkId: string
    subnet: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "subnet": "string",
      "applianceIp": "string",
      "mandatoryDhcp": {
        "enabled": true
      },
      "ipv6": {
        "enabled": true,
        "prefixAssignments": [
          {
            "autonomous": true,
            "staticPrefix": "string",
            "staticApplianceIp6": "string",
            "origin": {
              "type": "string",
              "interfaces": [
                "string"
              ]
            }
          }
        ]
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_cellular_gateway_port_forwarding_rules
short_description: Resource module for devices _cellular _gateway _port _forwarding
  _rules
description:
  - Manage operation update of the resource devices _cellular _gateway _port _forwarding
    _rules.
  - Updates the port forwarding rules for a single MG.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  rules:
    description: An array of port forwarding params.
    elements: dict
    suboptions:
      access:
        description: '`any` or `restricted`. Specify the right to make inbound connections
          on the specified ports or port ranges. If `restricted`, a list of allowed
          IPs is mandatory.'
        type: str
      allowedIps:
        description: An array of ranges of WAN IP addresses that are allowed to make
          inbound connections on the specified ports or port ranges.
        elements: str
        type: list
      lanIp:
        description: The IP address of the server or device that hosts the internal
          resource that you wish to make available on the WAN.
        type: str
      localPort:
        description: A port or port ranges that will receive the forwarded traffic
          from the WAN.
        type: str
      name:
        description: A descriptive name for the rule.
        type: str
      protocol:
        description: TCP or UDP.
        type: str
      publicPort:
        description: A port or port ranges that will be forwarded to the host on the
          LAN.
        type: str
    type: list
  serial:
    description: Serial path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for cellularGateway updateDeviceCellularGatewayPortForwardingRules
    description: Complete reference of the updateDeviceCellularGatewayPortForwardingRules
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-device-cellular-gateway-port-forwarding-rules
notes:
  - SDK Method used are
    cellular_gateway.CellularGateway.update_device_cellular_gateway_port_forwarding_rules,
  - Paths used are
    put /devices/{serial}/cellularGateway/portForwardingRules,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.devices_cellular_gateway_port_forwarding_rules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    rules:
      - access: any
        allowedIps:
          - 10.10.10.10
          - 10.10.10.11
        lanIp: 172.31.128.5
        localPort: '4'
        name: test
        protocol: tcp
        publicPort: 11-12
    serial: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    [
      {
        "name": "string",
        "lanIp": "string",
        "publicPort": "string",
        "localPort": "string",
        "allowedIps": [
          "string"
        ],
        "protocol": "string",
        "access": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_global_assignment
short_description: Manages global-assignment objects on Check Point over Web Services API
description:
  - Manages global-assignment objects on Check Point devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "1.0.0"
author: "Or Soffer (@chkp-orso)"
options:
  dependent_domain:
    description:
      - N/A
    type: str
  global_access_policy:
    description:
      - Global domain access policy that is assigned to a dependent domain.
    type: str
  global_domain:
    description:
      - N/A
    type: str
  global_threat_prevention_policy:
    description:
      - Global domain threat prevention policy that is assigned to a dependent domain.
    type: str
  manage_protection_actions:
    description:
      - N/A
    type: bool
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-global-assignment
  cp_mgmt_global_assignment:
    dependent_domain: domain2
    global_access_policy: standard
    global_domain: Global
    global_threat_prevention_policy: standard
    manage_protection_actions: true
    state: present

- name: set-global-assignment
  cp_mgmt_global_assignment:
    dependent_domain: domain1
    global_domain: Global2
    global_threat_prevention_policy: ''
    manage_protection_actions: false
    state: present

- name: delete-global-assignment
  cp_mgmt_global_assignment:
    dependent_domain: domain1
    global_domain: Global2
    state: absent
"""

RETURN = """
cp_mgmt_global_assignment:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        dependent_domain=dict(type="str"),
        global_access_policy=dict(type="str"),
        global_domain=dict(type="str"),
        global_threat_prevention_policy=dict(type="str"),
        manage_protection_actions=dict(type="bool"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "global-assignment"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The cisco.ios_ospf_interfaces config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.ospf_interfaces import (
    Ospf_interfacesTemplate,
)


class Ospf_interfaces(ResourceModule):
    """
    The cisco.ios_ospf_interfaces config class
    """

    def __init__(self, module):
        super(Ospf_interfaces, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="ospf_interfaces",
            tmplt=Ospf_interfacesTemplate(),
        )
        self.parsers = []

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """

        # turn all lists of dicts into dicts prior to merge
        wantd = self._list_to_dict(self.want, "want")
        haved = self._list_to_dict(self.have)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            wantd = {}

        # remove superfluous config for overridden and deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in haved.items():
                if k not in wantd:
                    self._compare(want={}, have=have, interface=k)

        for k, want in wantd.items():
            self._compare(want=want, have=haved.pop(k, {}), interface=k)

    def _compare(self, want, have, interface):
        begin = len(self.commands)
        self._compare_afis(want=want, have=have)
        if len(self.commands) != begin:
            self.commands.insert(begin, self._tmplt.render({"name": interface}, "name", False))

    def _compare_afis(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Ospf_interfaces network resource.
        """

        parsers = [
            "name",
            "process",
            "adjacency",
            "authentication",
            "bfd",
            "cost",
            "database_filter",
            "dead_interval",
            "demand_circuit",
            "flood_reduction",
            "hello_interval",
            "lls",
            "manet",
            "mtu_ignore",
            "multi_area",
            "neighbor",
            "network",
            "prefix_suppression",
            "priority",
            "resync_timeout",
            "retransmit_interval",
            "shutdown",
            "transmit_delay",
            "ttl_security",
        ]

        for afi in ("ipv4", "ipv6"):
            wacls = want.pop(afi, {})
            hacls = have.pop(afi, {})

            self.compare(parsers=parsers, want=wacls, have=hacls)

    def _list_to_dict(self, entry, attr_type=None):
        if self.state == "deleted" and attr_type == "want":
            del_list = {}
            for intf in entry:
                del_list[intf.get("name")] = {}
            return del_list

        list_to_dict = {}
        for intf in entry:
            if intf.get("address_family"):
                list_to_dict[intf.get("name")] = self.process_list_attr(intf)
        return list_to_dict

    def process_list_attr(self, add_fam):
        item = {}
        for ag in add_fam.get("address_family", []):
            item[ag.get("afi")] = ag
        return item



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_zigbee_devices_info
short_description: Information module for organizations _wireless _zigbee _devices
description:
  - Get all organizations _wireless _zigbee _devices.
  - List the Zigbee wireless devices for an organization or the supplied networks.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 10.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - >
        NetworkIds query parameter. Parameter of networks you want the zigbee devices
        for. E.g. NetworkIds=N_12345678&networkIds=N_3456.
    elements: str
    type: list
  isEnrolled:
    description:
      - IsEnrolled query parameter. Filter devices based on if they are enrolled or
        not.
    type: bool
  search:
    description:
      - Search query parameter. Filter devices by their name, tag or serial.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getOrganizationWirelessZigbeeDevices
    description: Complete reference of the getOrganizationWirelessZigbeeDevices API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-wireless-zigbee-devices
notes:
  - SDK Method used are
    wireless.Wireless.get_organization_wireless_zigbee_devices,
  - Paths used are
    get /organizations/{organizationId}/wireless/zigbee/devices,
"""

EXAMPLES = r"""
- name: Get all organizations _wireless _zigbee _devices
  cisco.meraki.organizations_wireless_zigbee_devices_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    isEnrolled: true
    search: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "network": {
          "id": "string",
          "name": "string"
        },
        "panId": "string",
        "channel": "string",
        "transmitPowerLevel": 0,
        "enrolled": true,
        "status": "string",
        "gateway": {
          "name": "string",
          "mac": "string",
          "serial": "string",
          "tags": [
            "string"
          ]
        },
        "counts": {
          "doorLocks": {
            "byStatus": {
              "online": 0,
              "offline": 0,
              "dormant": 0
            }
          }
        }
      }
    ]
"""



#!/usr/bin/python
#
# Copyright (c) 2019 Piotr Wojciechowski <piotr@it-playground.pl>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_node_info

short_description: Retrieves facts about docker swarm node from Swarm Manager

description:
  - Retrieves facts about a docker node.
  - Essentially returns the output of C(docker node inspect <name>).
  - Must be executed on a host running as Swarm Manager, otherwise the module will fail.
extends_documentation_fragment:
  - community.docker._docker
  - community.docker._docker.docker_py_2_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker
  - community.docker._attributes.info_module
  - community.docker._attributes.idempotent_not_modify_state

options:
  name:
    description:
      - The name of the node to inspect.
      - The list of nodes names to inspect.
      - If empty then return information of all nodes in Swarm cluster.
      - When identifying the node use either the hostname of the node (as registered in Swarm) or node ID.
      - If O(self=true) then this parameter is ignored.
    type: list
    elements: str
  self:
    description:
      - If V(true), queries the node (that is, the docker daemon) the module communicates with.
      - If V(true) then O(name) is ignored.
      - If V(false) then query depends on O(name) presence and value.
    type: bool
    default: false

author:
  - Piotr Wojciechowski (@WojciechowskiPiotr)

requirements:
  - "L(Docker SDK for Python,https://docker-py.readthedocs.io/en/stable/) >= 2.4.0"
  - "Docker API >= 1.25"
"""

EXAMPLES = r"""
---
- name: Get info on all nodes
  community.docker.docker_node_info:
  register: result

- name: Get info on node
  community.docker.docker_node_info:
    name: mynode
  register: result

- name: Get info on list of nodes
  community.docker.docker_node_info:
    name:
      - mynode1
      - mynode2
  register: result

- name: Get info on host if it is Swarm Manager
  community.docker.docker_node_info:
    self: true
  register: result
"""

RETURN = r"""
nodes:
  description:
    - Facts representing the current state of the nodes. Matches the C(docker node inspect) output.
    - Can contain multiple entries if more than one node provided in O(name), or O(name) is not provided.
    - If O(name) contains a list of nodes, the output will provide information on all nodes registered at the swarm, including
      nodes that left the swarm but have not been removed from the cluster on swarm managers and nodes that are unreachable.
  returned: always
  type: list
  elements: dict
"""

import traceback
import typing as t

from ansible_collections.community.docker.plugins.module_utils._common import (
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._swarm import (
    AnsibleDockerSwarmClient,
)

try:
    from docker.errors import DockerException
except ImportError:
    # missing Docker SDK for Python handled in ansible.module_utils.docker.common
    pass


def get_node_facts(client: AnsibleDockerSwarmClient) -> list[dict[str, t.Any]]:
    results: list[dict[str, t.Any]] = []

    if client.module.params["self"] is True:
        self_node_id = client.get_swarm_node_id()
        node_info = client.get_node_inspect(node_id=self_node_id)
        results.append(node_info)
        return results

    if client.module.params["name"] is None:
        node_info_list = client.get_all_nodes_inspect()
        return node_info_list

    nodes = client.module.params["name"]
    if not isinstance(nodes, list):
        nodes = [nodes]

    for next_node_name in nodes:
        next_node_info = client.get_node_inspect(
            node_id=next_node_name, skip_missing=True
        )
        if next_node_info:
            results.append(next_node_info)
    return results


def main() -> None:
    argument_spec = {
        "name": {"type": "list", "elements": "str"},
        "self": {"type": "bool", "default": False},
    }

    client = AnsibleDockerSwarmClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        min_docker_version="2.4.0",
    )

    client.fail_task_if_not_swarm_manager()

    try:
        nodes = get_node_facts(client)

        client.module.exit_json(
            changed=False,
            nodes=nodes,
        )
    except DockerException as e:
        client.fail(
            f"An unexpected docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when Docker SDK for Python tried to talk to the docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Fred-Sun, (@Fred-Sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_registrationdefinition
version_added: '1.3.0'
short_description: Manage Azure RegistrationDefinition instance
description:
    - Create, update and delete instance of Azure RegistrationDefinition.
options:
    registration_definition_id:
        description:
            - ID of the registration definition.
            - If is not specified, an UUID will be generated for it.
        type: str
    scope:
        description:
            - The subscription ID defines the subscription in which the registration definition will be created.
            - If not specified, will use the subscription derived from AzureRMAuth.
        type: str
    properties:
        description:
            - Properties of a registration definition.
        type: dict
        suboptions:
            description:
                description:
                    - Description of the registration definition.
                type: str
            authorizations:
                description:
                    - Authorization tuple containing principal ID of the user/security group or service principal and ID of the build-in role.
                required: true
                type: list
                elements: dict
                suboptions:
                    principal_id:
                        description:
                            - Principal ID of the security group/service principal/user that would be assigned permissions to the projected subscription.
                        required: true
                        type: str
                    role_definition_id:
                        description:
                            - The role definition identifier.
                            - This role will define all the permissions that the security group/service principal/user must have on the projected subscription.
                            - This role cannot be an owner role.
                        required: true
                        type: str
            registration_definition_name:
                description:
                    - Name of the registration definition.
                type: str
            managed_by_tenant_id:
                description:
                    - ID of the managedBy tenant.
                required: true
                type: str
    plan:
        description:
            - Plan details for the managed services.
        type: dict
        suboptions:
            name:
                description:
                    - The plan name.
                required: true
                type: str
            publisher:
                description:
                    - The publisher ID.
                required: true
                type: str
            product:
                description:
                    - The product code.
                required: true
                type: str
            version:
                description:
                    - The plan's version.
                required: true
                type: str
    state:
        description:
            - Assert the state of the RegistrationDefinition.
            - Use C(present) to create or update an RegistrationDefinition and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Fred-Sun (@Fred-Sun)

'''

EXAMPLES = '''
- name: Create Registration Definition without scope
  azure_rm_registrationdefinition:
    registration_definition_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    properties:
      description: test
      authorizations:
        - principal_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
          role_definition_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
      managed_by_tenant_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
      registration_definition_name: def4

- name: Create Registration Definition with scope
  azure_rm_registrationdefinition:
    scope: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    registration_definition_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    properties:
      description: test
      authorizations:
        - principal_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
          role_definition_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
      managed_by_tenant_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
      registration_definition_name: def5

- name: Delete Registration Definition
  azure_rm_registrationdefinition:
    registration_definition_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    state: absent
'''

RETURN = '''
state:
    description:
        - The state info of the registration assignment.
    type: complex
    returned: always
    contains:
        properties:
            description:
                - Properties of a registration definition.
            returned: always
            type: complex
            contains:
                description:
                    description:
                        - Description of the registration definition.
                    returned: always
                    type: str
                    sample: test
                authorizations:
                    description:
                        - Authorization tuple containing principal ID of the user/security group or service principal and ID of the build-in role.
                    returned: always
                    type: complex
                    contains:
                        principal_id:
                            description:
                                - Principal ID of the security group/service principal/user that would be assigned permissions to the projected subscription
                            returned: always
                            type: str
                            sample: 99e3227f-8701-4099-869f-bc3efc7f1e64
                        role_definition_id:
                            description:
                                - The role definition identifier.
                                - This role will define all the permissions that the security group/service principal/user must have on the subscription.
                                - This role cannot be an owner role.
                            returned: always
                            type: str
                            sample: b24988ac-6180-42a0-ab88-20f7382dd24c
                registration_definition_name:
                    description:
                        - Name of the registration definition.
                    returned: always
                    type: str
                    sample: null
                managed_by_tenant_id:
                    description:
                        - ID of the managedBy tenant.
                    returned: always
                    type: str
                    sample: null
        plan:
            description:
                - Plan details for the managed services.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - The plan name.
                    returned: always
                    type: str
                    sample: null
                publisher:
                    description:
                        - The publisher ID.
                    returned: always
                    type: str
                    sample: null
                product:
                    description:
                        - The product code.
                    returned: always
                    type: str
                    sample: null
                version:
                    description:
                        - The plan's version.
                    returned: always
                    type: str
                    sample: null
        id:
            description:
                - Fully qualified path of the registration definition.
            returned: always
            type: str
            sample: null
        type:
            description:
                - Type of the resource.
            returned: always
            type: str
            sample: Microsoft.ManagedServices/registrationDefinitions
        name:
            description:
                - Name of the registration definition.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/providers/Microsoft.ManagedServices/registrationDefinitions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx

'''
import uuid
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
try:
    from azure.mgmt.managedservices import ManagedServicesClient
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMRegistrationDefinition(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            scope=dict(
                type='str'
            ),
            registration_definition_id=dict(
                type='str',
            ),
            properties=dict(
                type='dict',
                options=dict(
                    description=dict(
                        type='str',
                    ),
                    authorizations=dict(
                        type='list',
                        required=True,
                        elements='dict',
                        options=dict(
                            principal_id=dict(
                                type='str',
                                required=True
                            ),
                            role_definition_id=dict(
                                type='str',
                                required=True
                            )
                        )
                    ),
                    registration_definition_name=dict(
                        type='str',
                    ),
                    managed_by_tenant_id=dict(
                        type='str',
                        required=True
                    )
                )
            ),
            plan=dict(
                type='dict',
                options=dict(
                    name=dict(
                        type='str',
                        required=True
                    ),
                    publisher=dict(
                        type='str',
                        required=True
                    ),
                    product=dict(
                        type='str',
                        required=True
                    ),
                    version=dict(
                        type='str',
                        required=True
                    )
                )
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.scope = None
        self.registration_definition_id = None
        self.body = {}

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction

        super(AzureRMRegistrationDefinition, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                            supports_check_mode=True,
                                                            supports_tags=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        if self.registration_definition_id is None:
            self.registration_definition_id = str(uuid.uuid4())

        if not self.scope:
            self.scope = "/subscriptions/" + self.subscription_id
        else:
            self.scope = "/subscriptions/" + self.scope

        old_response = None
        response = None

        self.mgmt_client = self.get_mgmt_svc_client(ManagedServicesClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager,
                                                    api_version='2022-10-01',
                                                    suppress_subscription_id=True)

        old_response = self.get_resource()

        if not old_response:
            if self.state == 'present':
                self.to_do = Actions.Create
        else:
            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                if self.body.get('plan') is not None:
                    if old_response.get('plan') is not None and \
                       not all(self.body['plan'][item] == old_response['plan'].get(item)
                       for item in self.body['plan'].keys()):
                        self.to_do = Actions.Update
                    else:
                        self.to_do = Actions.Update
                elif (self.body.get('properties') is not None and
                      not all(self.body['properties'][item] == old_response['properties'].get(item)
                      for item in self.body['properties'].keys())):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.create_update_resource()
            self.results['state'] = response
        elif self.to_do == Actions.Delete:
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            self.delete_resource()
        else:
            self.results['changed'] = False
            response = old_response

        return self.results

    def create_update_resource(self):

        try:
            response = self.mgmt_client.registration_definitions.begin_create_or_update(
                registration_definition_id=self.registration_definition_id,
                scope=self.scope,
                request_body=self.body)

            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.log('Error attempting to create the RegistrationDefinition instance.')
            self.fail('Error creating the RegistrationDefinition instance: {0}'.format(str(exc)))
        return response.as_dict()

    def delete_resource(self):
        try:
            response = self.mgmt_client.registration_definitions.delete(registration_definition_id=self.registration_definition_id,
                                                                        scope=self.scope)
        except Exception as e:
            self.log('Error attempting to delete the RegistrationDefinition instance.')
            self.fail('Error deleting the RegistrationDefinition instance: {0}'.format(str(e)))

        return True

    def get_resource(self):
        try:
            response = self.mgmt_client.registration_definitions.get(scope=self.scope,
                                                                     registration_definition_id=self.registration_definition_id)
        except Exception as e:
            return False
        return response.as_dict()


def main():
    AzureRMRegistrationDefinition()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Prefix_lists parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Prefix_listsTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Prefix_listsTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "entry",
            "getval": re.compile(
                r"""
                ^(?P<afi>ip|ipv6)
                \sprefix-list
                \s(?P<name>\S+)
                \sseq\s(?P<sequence>\d+)
                \s(?P<action>permit|deny)
                \s(?P<prefix>\S+)
                (\seq\s(?P<eq>\d+))?
                (\sge\s(?P<ge>\d+))?
                (\sle\s(?P<le>\d+))?
                (\smask\s(?P<mask>\S+))?
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "{{ 'ip' if afi == 'ipv4' else afi }} prefix-list {{ name }}"
                      "{{ (' seq ' + sequence|string) if sequence|d('') else '' }}"
                      " {{ action }}"
                      " {{ prefix }}"
                      "{{ (' eq ' + eq|string) if eq|d('') else '' }}"
                      "{{ (' ge ' + ge|string) if ge|d('') else '' }}"
                      "{{ (' le ' + le|string) if le|d('') else '' }}"
                      "{{ (' mask ' + mask) if mask|d('') else '' }}",
            "result": {
                "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}": {
                    "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                    "prefix_lists": {
                        "{{ name }}": {
                            "name": "{{ name }}",
                            "entries": [
                                {
                                    "sequence": "{{ sequence|d(None) }}",
                                    "action": "{{ action }}",
                                    "prefix": "{{ prefix }}",
                                    "eq": "{{ eq }}",
                                    "ge": "{{ ge }}",
                                    "le": "{{ le }}",
                                    "mask": "{{ mask }}",
                                },
                            ],
                        },
                    },
                },
            },
        },
        {
            "name": "description",
            "getval": re.compile(
                r"""
                ^(?P<afi>ip|ipv6)
                \sprefix-list
                \s(?P<name>\S+)
                \sdescription\s(?P<description>.+)\s*
                $""", re.VERBOSE,
            ),
            "setval": "{{ 'ip' if afi == 'ipv4' else afi }} prefix-list {{ name }} description {{ description }}",
            "result": {
                "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}": {
                    "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                    "prefix_lists": {
                        "{{ name }}": {
                            "name": "{{ name }}",
                            "description": "{{ description }}",
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l3out_logical_node_profile
short_description: Manage Layer 3 Outside (L3Out) logical node profiles (l3ext:LNodeP)
description:
- Manage Layer 3 Outside (L3Out) logical node profiles on Cisco ACI fabrics.
options:
  node_profile:
    description:
    - Name of the node profile.
    type: str
    aliases: [ node_profile_name, name, logical_node ]
  description:
    description:
    - Description for the node profile.
    type: str
    aliases: [ descr ]
  tenant:
    description:
    - Name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  l3out:
    description:
    - Name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
  dscp:
    description:
    - The target Differentiated Service (DSCP) value.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, unspecified ]
    aliases: [ target_dscp ]
  mpls_custom_qos_policy:
    description:
    - The MPLS custom QoS policy name for the node profile.
    - This argument should only be used for Infra SR-MPLS L3Outs.
    aliases: [ mpls_custom_qos_policy_name ]
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- module: cisco.aci.aci_l3out
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(l3ext:LNodeP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Jason Juenger (@jasonjuenger)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add a new node profile
  cisco.aci.aci_l3out_logical_node_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    node_profile: my_node_profile
    description: node profile for my_l3out
    l3out: my_l3out
    tenant: my_tenant
    dscp: CS0
    state: present
  delegate_to: localhost

- name: Add a new node profile with MPLS custom QOS policy to SR-MPLS infra l3out
  cisco.aci.aci_l3out_logical_node_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: infra
    l3out: infra_sr_mpls_l3out
    node_profile: infra_sr_mpls_l3out_node_profile
    mpls_custom_qos_policy: infra_mpls_custom_qos_policy
    state: present
  delegate_to: localhost

- name: Delete a node profile
  cisco.aci.aci_l3out_logical_node_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    node_profile: my_node_profile
    l3out: my_l3out
    tenant: my_tenant
    state: absent
  delegate_to: localhost

- name: Query a node profile
  cisco.aci.aci_l3out_logical_node_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    node_profile: my_node_profile
    l3out: my_l3out
    tenant: my_tenant
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all node profile for L3out
  cisco.aci.aci_l3out_logical_node_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    l3out: my_l3out
    tenant: my_tenant
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        node_profile=dict(type="str", aliases=["name", "node_profile_name", "logical_node"]),
        tenant=dict(type="str", aliases=["tenant_name"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        description=dict(type="str", aliases=["descr"]),
        dscp=dict(
            type="str",
            choices=[
                "AF11",
                "AF12",
                "AF13",
                "AF21",
                "AF22",
                "AF23",
                "AF31",
                "AF32",
                "AF33",
                "AF41",
                "AF42",
                "AF43",
                "CS0",
                "CS1",
                "CS2",
                "CS3",
                "CS4",
                "CS5",
                "CS6",
                "CS7",
                "EF",
                "VA",
                "unspecified",
            ],
            aliases=["target_dscp"],
        ),
        # alias=dict(type="str"), not implemented because of different (api/alias/mo/uni/) api endpoint
        mpls_custom_qos_policy=dict(type="str", aliases=["mpls_custom_qos_policy_name"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "l3out", "node_profile"]],
            ["state", "present", ["tenant", "l3out", "node_profile"]],
        ],
    )

    node_profile = module.params.get("node_profile")
    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    description = module.params.get("description")
    dscp = module.params.get("dscp")
    mpls_custom_qos_policy = module.params.get("mpls_custom_qos_policy")
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    aci = ACIModule(module)

    child_classes = []
    if mpls_custom_qos_policy is not None:
        child_classes.append("l3extRsLNodePMplsCustQosPol")

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extLNodeP",
            aci_rn="lnodep-{0}".format(node_profile),
            module_object=node_profile,
            target_filter={"name": node_profile},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if mpls_custom_qos_policy is not None:
            if mpls_custom_qos_policy == "":
                child_configs.append(dict(l3extRsLNodePMplsCustQosPol=dict(attributes=dict(status="deleted"))))
            else:
                child_configs.append(
                    dict(l3extRsLNodePMplsCustQosPol=dict(attributes=dict(tDn="uni/tn-infra/qosmplscustom-{0}".format(mpls_custom_qos_policy))))
                )

        aci.payload(
            aci_class="l3extLNodeP",
            class_config=dict(
                descr=description,
                name=node_profile,
                targetDscp=dscp,
                nameAlias=name_alias,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="l3extLNodeP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#
# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from ansible.errors import AnsibleConnectionFailure
from ansible_collections.ansible.netcommon.plugins.plugin_utils.terminal_base import TerminalBase


class TerminalModule(TerminalBase):
    terminal_stdout_re = [
        re.compile(rb"[\r\n]*[\w+\-\.:\/\[\]]+(?:\([^\)]+\)){,3}(?:>|#) ?$"),
        re.compile(rb"]]>]]>[\r\n]?"),
    ]

    terminal_stderr_re = [
        re.compile(rb"% ?Error"),
        re.compile(rb"% ?Bad secret"),
        re.compile(rb"% ?This command is not authorized"),
        re.compile(rb"invalid input", re.I),
        re.compile(rb"(?:incomplete|ambiguous) command", re.I),
        re.compile(rb"(?<!\()connection timed out(?!\))", re.I),
        re.compile(rb"[^\r\n]+ not found", re.I),
        re.compile(rb"'[^']' +returned error code: ?\d+"),
        re.compile(rb"Failed to commit", re.I),
        re.compile(rb"show configuration failed \[inheritance\]", re.I),
    ]

    terminal_config_prompt = re.compile(r"^.+\(config(-.*)?\)#$")

    def on_open_shell(self):
        try:
            for cmd in (
                b"terminal length 0",
                b"terminal width 512",
                b"terminal exec prompt no-timestamp",
            ):
                self._exec_cli_command(cmd)
        except AnsibleConnectionFailure:
            raise AnsibleConnectionFailure("unable to set terminal parameters")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l4l7_service_graph_template_connection_to_connector
version_added: "2.12.0"
short_description: Manage L4-L7 Service Graph Template Connections between function nodes and terminal nodes (vns:RsAbsConnectionConns)
description:
- Manage L4-L7 Service Graph Template Connections to define traffic flows between function nodes and terminal nodes.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  service_graph:
    description:
    - The name of an existing Service Graph.
    type: str
  connection_name:
    description:
    - The name of an existing vns:AbsConnection object.
    type: str
  direction:
    description:
    - The direction of the connection.
    - If this links to a terminal node, both vns:RsAbsConnectionConns will use the same direction.
    - Otherwise one vns:RsAbsConnectionConns will be consumer, and the other will be provider.
    type: str
    choices: [ consumer, provider ]
  connected_node:
    description:
    - The name of an existing node.
    - Omit this variable for connections to terminal nodes.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
notes:
- The I(tenant), I(service_graph) and I(connection_name) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant), M(cisco.aci.aci_l4l7_service_graph_template)
  and M(cisco.aci.aci_l4l7_service_graph_template_abs_conn) modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l4l7_service_graph_template
- module: cisco.aci.aci_l4l7_service_graph_template_abs_connection
- module: cisco.aci.aci_l4l7_service_graph_template_node
- name: APIC Management Information Model reference
  description: More information about the internal APIC class, B(vns:RsAbsConnectionConns)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Add a new connection
  cisco.aci.aci_l4l7_service_graph_template_connection_to_connector:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    service_graph: test-graph
    direction: provider
    connection_name: C1
    state: present
  delegate_to: localhost

- name: Query a connection
  cisco.aci.aci_l4l7_service_graph_template_connection_to_connector:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    service_graph: test-graph
    direction: provider
    connection_name: C1
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete a connection
  cisco.aci.aci_l4l7_service_graph_template_connection_to_connector:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    service_graph: test-graph
    direction: provider
    connection_name: C1
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        service_graph=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        connection_name=dict(type="str"),
        direction=dict(type="str", choices=["consumer", "provider"]),
        connected_node=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "service_graph", "connection_name", "direction"]],
            ["state", "present", ["tenant", "service_graph", "connection_name", "direction"]],
        ],
    )

    tenant = module.params.get("tenant")
    service_graph = module.params.get("service_graph")
    state = module.params.get("state")
    connection_name = module.params.get("connection_name")
    direction = module.params.get("direction")
    connected_node = module.params.get("connected_node")

    aci = ACIModule(module)
    if connected_node:
        tdn = "uni/tn-{0}/AbsGraph-{1}/AbsNode-{2}/AbsFConn-{3}".format(tenant, service_graph, connected_node, direction)
    elif direction == "consumer":
        tdn = "uni/tn-{0}/AbsGraph-{1}/AbsTermNodeCon-T1/AbsTConn".format(tenant, service_graph)
    elif direction == "provider":
        tdn = "uni/tn-{0}/AbsGraph-{1}/AbsTermNodeProv-T2/AbsTConn".format(tenant, service_graph)

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="vnsAbsGraph",
            aci_rn="AbsGraph-{0}".format(service_graph),
            module_object=service_graph,
            target_filter={"name": service_graph},
        ),
        subclass_2=dict(
            aci_class="vnsAbsConnection",
            aci_rn="AbsConnection-{0}".format(connection_name),
            module_object=connection_name,
            target_filter={"name": connection_name},
        ),
        subclass_3=dict(
            aci_class="vnsRsAbsConnectionConns",
            aci_rn="rsabsConnectionConns-[{0}]".format(tdn),
            module_object=tdn,
            target_filter={"tDn": tdn},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="vnsRsAbsConnectionConns",
            class_config=dict(tDn=tdn),
        )
        aci.get_diff(aci_class="vnsRsAbsConnectionConns")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_set_trust
short_description: Configure a Trusted communication between the Management Server and the managed Security Gateway.
description:
  - Configure a Trusted communication between the Management Server and the managed Security Gateway.
  - All operations are performed over Web Services API.
  - Available from R82.10 management version.
version_added: "6.7.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
  ipv4_address:
    description:
      - IP address of the object, for establishing trust with dynamic gateways.
    type: str
  one_time_password:
    description:
      - Shared password to establish SIC between the Security Management and the Security Gateway.
    type: str
  trust_method:
    description:
      - Establish the trust communication method.
    type: str
    choices: ['one_time_password', 'without_password_not_secure', 'cloud_token']
  trust_settings:
    description:
      - Settings for the trusted communication establishment.
    type: dict
    suboptions:
      gateway_mac_address:
        description:
          - Use the Security Gateway MAC address, relevant for the gateway_mac_address identification-method.
        type: str
      identification_method:
        description:
          - How to identify the gateway (relevant for Spark DAIP gateways only).
        type: str
        choices: ['gateway_name', 'mac_address', 'none_not_secure', 'ip_address']
      initiation_phase:
        description:
          - Push the certificate to the Security Gateway immediately, or wait for the Security Gateway to pull the certificate. Default value for
            Spark Gateway is 'when_gateway_connects'.
        type: str
        choices: ['now', 'when_gateway_connects']
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: set-trust
  cp_mgmt_set_trust:
    name: gw1
    one_time_password: aaaa
"""

RETURN = """
cp_mgmt_set_trust:
  description: The checkpoint set-trust output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        name=dict(type='str'),
        ipv4_address=dict(type='str'),
        one_time_password=dict(type='str', no_log=True),
        trust_method=dict(type='str', choices=['one_time_password', 'without_password_not_secure', 'cloud_token']),
        trust_settings=dict(type='dict', options=dict(
            gateway_mac_address=dict(type='str'),
            identification_method=dict(type='str', choices=['gateway_name', 'mac_address', 'none_not_secure', 'ip_address']),
            initiation_phase=dict(type='str', choices=['now', 'when_gateway_connects'])
        )),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "set-trust"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_server_profile_derive
short_description: Derive Server Profiles from Templates in Cisco Intersight
description:
  - Derive new Server Profiles from a Server Profile Template using the Intersight MoCloner API.
  - Sync existing derived profiles back to their template when the template has changed using the MoMerger API.
  - Idempotent - will not re-derive if the profile already exists, and will not sync if the profile is already in sync.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the Organization this resource is assigned to.
    type: str
    default: default
  template_name:
    description:
      - Name of the Server Profile Template to derive from.
      - The template must exist in the specified organization.
    type: str
    required: true
  profile_names:
    description:
      - List of Server Profile names to derive from the template.
      - Each name must be unique within the organization.
    type: list
    elements: str
    required: true
  state:
    description:
      - If C(present), will derive profiles that do not exist and sync profiles that are out of sync with the template.
      - If C(absent), will delete derived profiles.
    type: str
    choices: [present, absent]
    default: present
  force_sync:
    description:
      - When C(true), always sync existing derived profiles to the template, even if the profile appears to be in sync.
      - When C(false), only sync profiles whose ConfigContext shows they are out of sync with the template.
    type: bool
    default: false
author:
  - Steve Fulmer (@stevefulme1)
'''

EXAMPLES = r'''
- name: Derive a single server profile from a template
  cisco.intersight.intersight_server_profile_derive:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: default
    template_name: My-Standard-Template
    profile_names:
      - SP-Derived-1

- name: Derive multiple server profiles from a template
  cisco.intersight.intersight_server_profile_derive:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: default
    template_name: My-Standard-Template
    profile_names:
      - SP-Derived-1
      - SP-Derived-2
      - SP-Derived-3

- name: Force sync derived profiles back to template
  cisco.intersight.intersight_server_profile_derive:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    template_name: My-Standard-Template
    profile_names:
      - SP-Derived-1
    force_sync: true

- name: Delete derived server profiles
  cisco.intersight.intersight_server_profile_derive:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    template_name: My-Standard-Template
    profile_names:
      - SP-Derived-1
    state: absent
'''

RETURN = r'''
api_response:
  description: Summary of derive/sync operations performed.
  returned: always
  type: dict
  sample:
    "api_response": {
        "derived": ["SP-Derived-1", "SP-Derived-2"],
        "synced": ["SP-Derived-3"],
        "skipped": [],
        "deleted": []
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def get_profile_by_name(intersight, profile_name):
    """Look up a server profile by name."""
    options = {
        'http_method': 'get',
        'resource_path': '/server/Profiles',
        'query_params': {
            '$filter': "Name eq '" + profile_name + "'",
        },
    }
    response = intersight.call_api(**options)
    if response.get('Results'):
        return response['Results'][0]
    return None


def get_template_moid(intersight, template_name, organization_moid):
    """Look up a server profile template by name and org."""
    options = {
        'http_method': 'get',
        'resource_path': '/server/ProfileTemplates',
        'query_params': {
            '$filter': "Name eq '" + template_name + "' and Organization.Moid eq '" + organization_moid + "'",
        },
    }
    response = intersight.call_api(**options)
    if response.get('Results'):
        return response['Results'][0]['Moid']
    return None


def derive_profile(intersight, template_moid, profile_name, organization_moid):
    """Derive a new server profile from a template using MoCloner."""
    options = {
        'http_method': 'post',
        'resource_path': '/bulk/MoCloners',
        'body': {
            'Sources': [
                {
                    'ClassId': 'mo.MoRef',
                    'ObjectType': 'server.ProfileTemplate',
                    'Moid': template_moid,
                }
            ],
            'Targets': [
                {
                    'Name': profile_name,
                    'ObjectType': 'server.Profile',
                    'ClassId': 'server.Profile',
                    'Organization': {
                        'Moid': organization_moid,
                    },
                }
            ],
        },
    }
    return intersight.call_api(**options)


def sync_profile(intersight, template_moid, profile_moid):
    """Sync a derived profile back to its template using MoMerger."""
    options = {
        'http_method': 'post',
        'resource_path': '/bulk/MoMergers',
        'body': {
            'Sources': [
                {
                    'ObjectType': 'server.ProfileTemplate',
                    'Moid': template_moid,
                }
            ],
            'Targets': [
                {
                    'ObjectType': 'server.Profile',
                    'Moid': profile_moid,
                }
            ],
            'MergeAction': 'Replace',
        },
    }
    return intersight.call_api(**options)


def delete_profile(intersight, profile_moid):
    """Delete a server profile by Moid."""
    options = {
        'http_method': 'delete',
        'resource_path': '/server/Profiles',
        'moid': profile_moid,
    }
    return intersight.call_api(**options)


def profile_needs_sync(profile, force_sync):
    """Check if a derived profile needs syncing."""
    if force_sync:
        return True
    config_context = profile.get('ConfigContext', {})
    config_state = config_context.get('ConfigState', '')
    return config_state in ('Pending-changes', 'Out-of-sync', 'Inconsistent')


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str', default='default'),
        template_name=dict(type='str', required=True),
        profile_names=dict(type='list', elements='str', required=True),
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        force_sync=dict(type='bool', default=False),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    organization_name = module.params['organization']
    template_name = module.params['template_name']
    profile_names = module.params['profile_names']
    state = module.params['state']
    force_sync = module.params['force_sync']

    organization_moid = intersight.get_moid_by_name(
        resource_path='/organization/Organizations',
        resource_name=organization_name,
    )
    if not organization_moid:
        module.fail_json(msg="Organization '%s' not found." % organization_name)

    result_summary = {
        'derived': [],
        'synced': [],
        'skipped': [],
        'deleted': [],
    }

    template_moid = None
    if state == 'present':
        template_moid = get_template_moid(intersight, template_name, organization_moid)
        if not template_moid:
            module.fail_json(msg="Server Profile Template '%s' not found in organization '%s'." % (template_name, organization_name))

    for profile_name in profile_names:
        existing_profile = get_profile_by_name(intersight, profile_name)

        if state == 'absent':
            if existing_profile:
                if not module.check_mode:
                    delete_profile(intersight, existing_profile['Moid'])
                result_summary['deleted'].append(profile_name)
                intersight.result['changed'] = True
            else:
                result_summary['skipped'].append(profile_name)
            continue

        if existing_profile:
            if profile_needs_sync(existing_profile, force_sync):
                if not module.check_mode:
                    sync_profile(intersight, template_moid, existing_profile['Moid'])
                result_summary['synced'].append(profile_name)
                intersight.result['changed'] = True
            else:
                result_summary['skipped'].append(profile_name)
        else:
            if not module.check_mode:
                derive_profile(intersight, template_moid, profile_name, organization_moid)
            result_summary['derived'].append(profile_name)
            intersight.result['changed'] = True

    intersight.result['api_response'] = result_summary
    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Tim Cragg (@timcragg)
# Copyright: (c) 2025, Shreyas Srish (@shrsr)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l4l7_service_graph_template
version_added: "2.12.0"
short_description: Manage L4-L7 Service Graph Templates (vns:AbsGraph)
description:
- Manage Layer 4 to Layer 7 (L4-L7) Service Graph Templates on Cisco ACI fabrics.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  service_graph:
    description:
      - The name of Service Graph Template.
    type: str
  ui_template_type:
    description:
      - The UI Template Type.
      - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [
      ndo_implicit_template,
      one_node_adc_one_arm,
      one_node_adc_one_arm_l3ext,
      one_node_adc_two_arm,
      one_node_fw_routed,
      one_node_fw_trans,
      two_node_fw_routed_adc_one_arm,
      two_node_fw_routed_adc_one_arm_l3ext,
      two_node_fw_routed_adc_two_arm,
      two_node_fw_trans_adc_one_arm,
      two_node_fw_trans_adc_one_arm_l3ext,
      two_node_fw_trans_adc_two_arm,
      unspecified
    ]
  type:
    description:
    - Specifies the type of Service Graph Template.
    - The APIC defaults to C(legacy) when unset during creation.
    type: str
    choices: [ cloud, legacy ]
  service_rule_type:
    description:
    - Defines the type of service rule applied within the Service Graph Template.
    - The APIC defaults to C(vrf) when unset during creation.
    type: str
    choices: [ epg, subnet, vrf ]
  filter_between_nodes:
    description:
    - Determines how traffic is filtered between nodes in the Service Graph Template.
    - The APIC defaults to C(allow-all) when unset during creation.
    type: str
    choices: [ allow-all, filters-from-contract ]
  description:
    description:
    - A description of the Service Graph Template.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner
notes:
- The I(tenant) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) module can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(vns:AbsGraph)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Add a new L4-L7 Service Graph Template
  cisco.aci.aci_l4l7_service_graph_template:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    service_graph: my_service_graph
    state: present
  delegate_to: localhost

- name: Query a Service Graph Template
  cisco.aci.aci_l4l7_service_graph_template:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    service_graph: my_service_graph
    state: query
  register: query_result
  delegate_to: localhost

- name: Query all Service Graph Templates
  cisco.aci.aci_l4l7_service_graph_template:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete a Service Graph Template
  cisco.aci.aci_l4l7_service_graph_template:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    service_graph: my_service_graph
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import L4L7_UI_TEMPLATE_TYPE


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        service_graph=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        ui_template_type=dict(type="str", choices=list(L4L7_UI_TEMPLATE_TYPE)),
        type=dict(type="str", choices=["cloud", "legacy"]),
        service_rule_type=dict(type="str", choices=["epg", "subnet", "vrf"]),
        filter_between_nodes=dict(type="str", choices=["allow-all", "filters-from-contract"]),
        description=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "service_graph"]],
            ["state", "present", ["tenant", "service_graph"]],
        ],
    )

    tenant = module.params.get("tenant")
    service_graph = module.params.get("service_graph")
    state = module.params.get("state")
    ui_template_type = L4L7_UI_TEMPLATE_TYPE.get(module.params.get("ui_template_type"))
    type = module.params.get("type")
    service_rule_type = module.params.get("service_rule_type")
    filter_between_nodes = module.params.get("filter_between_nodes")
    description = module.params.get("description")

    aci = ACIModule(module)

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="vnsAbsGraph",
            aci_rn="AbsGraph-{0}".format(service_graph),
            module_object=service_graph,
            target_filter={"name": service_graph},
        ),
        child_classes=["vnsAbsTermNodeProv", "vnsAbsTermNodeCon"],
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="vnsAbsGraph",
            class_config=dict(
                name=service_graph,
                uiTemplateType=ui_template_type,
                type=type,
                svcRuleType=service_rule_type,
                filterBetweenNodes=filter_between_nodes,
                descr=description,
            ),
        )
        aci.get_diff(aci_class="vnsAbsGraph")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2026, Jean Khawand (@jeankhawand)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: proton_pass
author:
  - Jean Khawand (@jeankhawand)
version_added: "13.2.0"
short_description: Fetch secrets from Proton Pass via the C(pass-cli) command-line tool.
description:
  - Retrieves secrets stored in Proton Pass by calling the C(pass-cli) binary
    on the Ansible control machine.
  - When only one item title is given, returns a dictionary of all extra fields for
    that item so the caller can access individual secrets by key.
  - When a field name is also given, returns the field value as a string directly,
    without parsing JSON.
  - Every lookup invokes C(pass-cli) directly — no in-process caching — so each
    call always reflects the current vault state.
requirements:
  - C(pass-cli) installed on the Ansible control machine
    (see U(https://github.com/protonpass/pass-cli))
notes:
  - Two non-interactive authentication paths are available.
  - B(Agent tokens) (recommended for CI/CD) are scoped to specific vaults,
    carry an expiration date, and produce an encrypted audit log — create one
    with C(pass-cli agent create <NAME> --expiration 1m --vault <VAULT>), then
    supply the token via O(pat) / E(ANSIBLE_PROTON_PASS_PAT) and set
    O(agent_reason) / E(ANSIBLE_PROTON_PASS_AGENT_REASON) to a short description
    of why the automation is accessing the vault.
  - B(Personal Access Tokens) B(PAT) grant full account access and should be
    reserved for interactive or development use — generate one in the Proton
    Pass web app under B(Account > Security > Access and authentication >
    Personal Access Tokens). PAT format is C(pst_<token>::<key>).
  - Authenticate once interactively with C(pass-cli login) before running
    playbooks when neither an agent token nor a PAT is configured.
  - Set O(debug=true) on the lookup call to display the raw C(pass-cli) JSON via
    Ansible's warning output — useful for diagnosing field extraction failures
    across C(pass-cli) versions.
  - >-
    All Proton Pass item types are supported: B(Login), B(Note), B(Credit Card),
    B(Wi-Fi), B(Identity), B(SSH Key), B(Custom), and B(Alias). Standard fields
    for each type (for example C(username)/C(password) for Login, C(ssid)/C(password)
    for Wi-Fi, C(private_key)/C(public_key) for SSH Key) are included in the returned
    dictionary alongside any custom extra fields.
  - For B(Login) items the C(urls) field is returned as a list of strings.
  - In Proton Pass, create one item per host. Add every secret as an extra B(hidden)
    field whose name exactly matches the key your playbooks reference (for example
    C(api_token), C(gpg_keyphrase)).
  - Sections (C(extra_sections)) are supported — fields nested inside a section
    are merged into the same flat dict as top-level extra fields.
options:
  _terms:
    description:
      - One or more Proton Pass item titles to look up.
      - If exactly two terms are given and O(field) is not set, the second term
        is interpreted as a field name and a plain string is returned instead of
        a dictionary.
    required: true
    type: list
    elements: str
  field:
    description:
      - Field name to retrieve. When set, the lookup returns a plain string
        instead of a dictionary, equivalent to passing the field as the second
        positional term.
      - When both a second positional term and O(field) are supplied, O(field)
        takes precedence.
    type: str
    default: ""
  vault:
    description:
      - Name of the Proton Pass vault to query.
      - When omitted, C(pass-cli) searches across all vaults accessible to
        the authenticated account; this may raise an error when the item title
        is not unique across vaults.
    type: str
    default: ""
    ini:
      - section: proton_pass_lookup
        key: vault
    env:
      - name: ANSIBLE_PROTON_PASS_VAULT
    vars:
      - name: ansible_proton_pass_vault
  pat:
    description:
      - Personal Access Token used for non-interactive authentication.
      - Format is C(pst_<token>::<key>).
      - When provided, the plugin runs C(pass-cli login --pat <value>) before
        the first item lookup if no active session is detected.
    type: str
    default: ""
    ini:
      - section: proton_pass_lookup
        key: pat
    env:
      - name: ANSIBLE_PROTON_PASS_PAT
    vars:
      - name: ansible_proton_pass_pat
  agent_reason:
    description:
      - Human-readable reason string required by Proton Pass when authenticating
        with an B(agent token) (as opposed to a personal account PAT).
      - Maps to the C(PROTON_PASS_AGENT_REASON) environment variable consumed by
        C(pass-cli). Must be between 1 and 300 characters.
      - Agent tokens are the recommended authentication mechanism for CI/CD
        pipelines and automated processes — they are scoped to specific vaults,
        carry an expiration date, and produce an encrypted audit log.
        Create one with C(pass-cli agent create <NAME> --expiration 1m
        --vault <VAULT_NAME>).
    type: str
    default: ""
    ini:
      - section: proton_pass_lookup
        key: agent_reason
    env:
      - name: ANSIBLE_PROTON_PASS_AGENT_REASON
    vars:
      - name: ansible_proton_pass_agent_reason
  cli_path:
    description:
      - Path to the C(pass-cli) binary.
      - When not set, C(pass-cli) is resolved from E(PATH).
    type: str
    default: pass-cli
    ini:
      - section: proton_pass_lookup
        key: cli_path
    env:
      - name: ANSIBLE_PROTON_PASS_CLI_PATH
    vars:
      - name: ansible_proton_pass_cli_path
  timeout:
    description:
      - Timeout in seconds for each C(pass-cli) subprocess invocation.
    type: int
    default: 30
    ini:
      - section: proton_pass_lookup
        key: timeout
    env:
      - name: ANSIBLE_PROTON_PASS_TIMEOUT
    vars:
      - name: ansible_proton_pass_timeout
  debug:
    description:
      - When C(true), displays the raw C(pass-cli) JSON via Ansible's warning output.
      - Useful for diagnosing field extraction failures across C(pass-cli) versions.
      - >-
        B(Note) that this can show sensitive information. Use only when
        absolutely necessary and ensure the sensitive output is not logged.
    type: bool
    default: false
"""

EXAMPLES = r"""
# Prerequisites — authenticate once on the control machine:
#   pass-cli login --pat pst_<token>::<key>
# Or export for non-interactive / CI use:
#   export ANSIBLE_PROTON_PASS_PAT='pst_<token>::<key>'
# Or configure ansible.cfg (applies to all lookups in the project):
#   [proton_pass_lookup]
#   vault = myproject

# --- All-fields mode: returns a dict of all extra fields ---

- name: Load all secrets for the current host
  ansible.builtin.set_fact:
    host_secrets: "{{ lookup('community.general.proton_pass', inventory_hostname, vault='myproject') }}"

- name: Use a field from the loaded dict
  ansible.builtin.debug:
    msg: "Token: {{ host_secrets.api_token }}"

# --- Single-field mode: returns a plain string ---

- name: Fetch one field via second positional term
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.proton_pass', 'my_host', 'api_token', vault='myproject') }}"

- name: Fetch one field via keyword argument
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.proton_pass', 'my_host', field='api_token', vault='myproject') }}"

# --- Multiple items, same field ---

- name: Fetch gpg_keyphrase from two items
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.proton_pass', 'host_a', 'host_b', field='gpg_keyphrase', vault='myproject') }}"

# --- Drop-in replacement for ansible-vault secrets ---
# Place in group_vars/all/secrets.yaml (plain YAML, safe to commit):
#
#   secrets:
#     host_a: "{{ lookup('community.general.proton_pass', 'host_a', vault='myproject') }}"
#     host_b: "{{ lookup('community.general.proton_pass', 'host_b', vault='myproject') }}"
#
# Existing role tasks continue to work without modification:
#   "{{ secrets[inventory_hostname].some_secret }}"

# --- Vault configured globally in ansible.cfg — no vault= kwarg needed ---
- name: Lookup when ansible.cfg sets [proton_pass_lookup] vault
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.proton_pass', 'my_host', 'api_token') }}"

# --- Agent token authentication (recommended for CI/CD) ---
# Create the agent first (one-time setup):
#   pass-cli agent create ci-runner --expiration 1m --vault myproject
# Then configure ansible.cfg:
#   [proton_pass_lookup]
#   vault = myproject
#   agent_reason = ansible playbook run
# And set the token in the environment (or ansible.cfg):
#   export ANSIBLE_PROTON_PASS_PAT='<agent-token>'
- name: Lookup using an agent token
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.proton_pass', 'my_host', 'api_token',
             pat=lookup('env', 'ANSIBLE_PROTON_PASS_PAT'),
             agent_reason='nightly deployment pipeline') }}"
"""

RETURN = r"""
_raw:
  description:
    - A list with one element per input term.
    - In single-field mode (O(field) set or two-term shorthand), each element is
      the plain-text value of the requested field.
    - In all-fields mode, each element is a flat dictionary mapping standard
      item-type fields (C(username)/C(password) for login, C(ssid)/C(password) for
      wi-fi, C(private_key)/C(public_key) for ssh-key, etc.) and every custom extra
      field to its string value.
  type: list
  elements: raw
"""

import json
import os
from subprocess import DEVNULL, PIPE, Popen, TimeoutExpired

from ansible.errors import AnsibleLookupError, AnsibleOptionsError
from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.parsing.convert_bool import boolean
from ansible.plugins.lookup import LookupBase
from ansible.utils.display import Display

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

display = Display()

# ---------------------------------------------------------------------------
# Item-type field registries — mirrors pass-cli item create <type> --get-template
# ---------------------------------------------------------------------------

_LOGIN_FIELDS = ("username", "email", "password", "totp_uri")
_CREDIT_CARD_FIELDS = ("cardholder_name", "card_type", "number", "verification_number", "expiration_date", "pin")
_WIFI_FIELDS = ("ssid", "password", "security")
_SSH_KEY_FIELDS = ("private_key", "public_key", "fingerprint", "key_type")
_IDENTITY_FIELDS = (
    "full_name",
    "email",
    "phone_number",
    "first_name",
    "middle_name",
    "last_name",
    "birthdate",
    "gender",
    "organization",
    "street_address",
    "zip_or_postal_code",
    "city",
    "state_or_province",
    "country_or_region",
    "floor",
    "county",
    "social_security_number",
    "passport_number",
    "license_number",
    "website",
    "x_handle",
    "second_phone_number",
    "linkedin",
    "reddit",
    "facebook",
    "yahoo",
    "instagram",
    "company",
    "job_title",
    "personal_website",
    "work_phone_number",
    "work_email",
)


class ProtonPassCLIError(AnsibleLookupError):
    """Raised when the pass-cli subprocess fails in an unexpected way."""


# ---------------------------------------------------------------------------
# CLI wrapper
# ---------------------------------------------------------------------------


class ProtonPassClient:
    """Thin wrapper around the pass-cli binary."""

    def __init__(
        self,
        cli_path: str,
        timeout: int,
        agent_reason: str,
        debug: bool = False,
    ) -> None:
        self.cli_path = cli_path
        self.timeout = timeout
        self.agent_reason = agent_reason
        self.debug = debug
        self._session_ok = False

    def _run(self, args: list[str]) -> tuple[int, str, str]:
        """Run pass-cli with *args* and return (returncode, stdout, stderr).

        When ``agent_reason`` is set, PROTON_PASS_AGENT_REASON is injected into
        the subprocess environment so that pass-cli records the reason in its
        encrypted audit log.

        Raises:
            ProtonPassCLIError: if the binary is not found or the call times out.
        """
        command = [self.cli_path] + args
        display.vvv(f"proton_pass: running {command}")

        env = os.environ.copy()
        if self.agent_reason:
            env["PROTON_PASS_AGENT_REASON"] = self.agent_reason

        try:
            proc = Popen(command, stdin=DEVNULL, stdout=PIPE, stderr=PIPE, env=env)
            out_bytes, err_bytes = proc.communicate(timeout=self.timeout)
            rc = proc.returncode
        except FileNotFoundError as e:
            raise ProtonPassCLIError(
                f"'{self.cli_path}' not found — install pass-cli or set "
                "ANSIBLE_PROTON_PASS_CLI_PATH / [proton_pass_lookup] cli_path "
                "in ansible.cfg"
            ) from e
        except TimeoutExpired as e:
            proc.kill()
            raise ProtonPassCLIError(f"pass-cli timed out after {self.timeout}s") from e

        return (
            rc,
            to_text(out_bytes, errors="surrogate_or_strict"),
            to_text(err_bytes, errors="surrogate_or_strict"),
        )

    # --- authentication ----------------------------------------------------------

    def test_session(self) -> bool:
        """Return True when an active pass-cli session exists."""
        rc, *_ignored = self._run(["test"])
        return rc == 0

    def login_with_pat(self, pat: str) -> None:
        """Authenticate using a Personal Access Token (``pst_<token>::<key>``)."""
        rc, _out, err = self._run(["login", "--pat", pat])
        if rc != 0:
            raise AnsibleLookupError(
                f"pass-cli login failed: {err.strip()} — verify the PAT format is pst_<token>::<key>"
            )

    def ensure_authenticated(self, pat: str) -> None:
        """Ensure there is an active session, logging in with PAT if necessary."""
        if self._session_ok:
            return
        if self.test_session():
            self._session_ok = True
            return
        if not pat:
            raise AnsibleLookupError(
                "No active pass-cli session and no PAT supplied. "
                "Run 'pass-cli login' on the control machine, or set "
                "ANSIBLE_PROTON_PASS_PAT."
            )
        self.login_with_pat(pat)
        self._session_ok = True

    # --- single-field retrieval --------------------------------------------------

    def fetch_all_fields(self, vault: str, title: str) -> dict[str, str | list[str]]:
        """Return all fields for *title* as a flat ``{field_name: value}`` dict.

        Always calls pass-cli — no caching, so callers always receive the
        current vault state.
        """
        args = ["item", "view", "--item-title", title, "--output", "json"]
        if vault:
            args += ["--vault-name", vault]

        rc, out, err = self._run(args)
        if rc != 0:
            _raise_item_error(title, vault, None, err)

        if self.debug:
            display.warning(f"proton_pass DEBUG — raw JSON for '{title}':\n{out}")

        try:
            raw = json.loads(out)
        except json.JSONDecodeError as e:
            raise AnsibleLookupError(
                f"pass-cli returned invalid JSON for item '{title}': {e}. "
                "Set the lookup option debug=true to inspect the raw output."
            ) from e

        return _parse_fields(raw, title)

    def fetch_field(self, vault: str, title: str, field: str) -> str:
        """Return a single field value for *title*.

        Delegates to ``fetch_all_fields`` so that trashed-item detection,
        JSON parsing, and error messages are consistent regardless of whether
        the caller requests one field or all fields.
        """
        all_fields = self.fetch_all_fields(vault=vault, title=title)
        if field not in all_fields:
            vault_hint = f" in vault '{vault}'" if vault else ""
            raise AnsibleLookupError(
                f"proton_pass: field '{field}' not found in item '{title}'{vault_hint}. "
                f"Available fields: {', '.join(sorted(all_fields)) or '(none)'}"
            )
        value = all_fields[field]
        return value if isinstance(value, str) else ", ".join(value)


# ---------------------------------------------------------------------------
# Error helper
# ---------------------------------------------------------------------------


def _raise_item_error(title: str, vault: str, field: str | None, stderr: str) -> None:
    """Translate pass-cli stderr into a descriptive AnsibleLookupError."""
    msg = stderr.strip()
    vault_hint = f" in vault '{vault}'" if vault else ""
    field_hint = f", field '{field}'" if field else ""

    lower = msg.lower()
    if "not found" in lower or "no items found" in lower:
        raise AnsibleLookupError(
            f"Item '{title}'{vault_hint} not found in Proton Pass{field_hint}. Verify the item title and vault name."
        )
    if "unauthorized" in lower or "unauthenticated" in lower:
        raise AnsibleLookupError(f"pass-cli authentication error: {msg}. Re-run 'pass-cli login' or refresh your PAT.")
    raise AnsibleLookupError(f"pass-cli failed for item '{title}'{vault_hint}{field_hint}: {msg}")


# ---------------------------------------------------------------------------
# JSON parser
# ---------------------------------------------------------------------------


def _extract_scalar_fields(source: dict, keys: tuple) -> dict[str, str]:
    """Return a dict of *keys* from *source*, skipping None and empty-string values."""
    return {key: str(val) for key in keys if (val := source.get(key)) is not None and val != ""}


def _extract_typed_fields(type_key: str, type_data: dict) -> dict[str, str | list[str]]:
    """Return type-specific standard fields extracted from *type_data*."""
    fields: dict[str, str | list[str]] = {}
    tk = type_key.lower()
    if tk == "login":
        fields.update(_extract_scalar_fields(type_data, _LOGIN_FIELDS))
        urls = type_data.get("urls")
        if urls:
            fields["urls"] = urls if isinstance(urls, list) else [urls]
    elif tk == "creditcard":
        fields.update(_extract_scalar_fields(type_data, _CREDIT_CARD_FIELDS))
    elif tk == "wifi":
        fields.update(_extract_scalar_fields(type_data, _WIFI_FIELDS))
    elif tk == "sshkey":
        fields.update(_extract_scalar_fields(type_data, _SSH_KEY_FIELDS))
        for section in type_data.get("sections", []):
            fields.update(_extract_extra_fields(section.get("extra_fields", [])))
    elif tk == "identity":
        fields.update(_extract_scalar_fields(type_data, _IDENTITY_FIELDS))
        for sub_key in (
            "extra_personal_details",
            "extra_address_details",
            "extra_contact_details",
            "extra_work_details",
        ):
            fields.update(_extract_extra_fields(type_data.get(sub_key, [])))
        for section in type_data.get("extra_sections", []):
            fields.update(_extract_extra_fields(section.get("extra_fields", [])))
    elif tk == "alias":
        for alias_key in ("aliased_email", "aliased_address"):
            val = type_data.get(alias_key)
            if val:
                fields["aliased_email"] = val
                break
    return fields


def _parse_fields(raw: dict, title: str) -> dict[str, str | list[str]]:
    """Extract a flat ``{field_name: value}`` dict from a pass-cli JSON response."""
    fields: dict[str, str | list[str]] = {}

    item_obj = raw.get("item", raw)
    state = item_obj.get("state", "")
    if state.lower() == "trashed":
        raise AnsibleLookupError(
            f"proton_pass: item '{title}' is in the Proton Pass trash and cannot be "
            "used. Restore it or permanently delete it."
        )

    content_wrapper = item_obj.get("content", {})
    note = content_wrapper.get("note")
    if note:
        fields["note"] = str(note)

    type_dict = content_wrapper.get("content")
    if isinstance(type_dict, dict):
        for type_key, type_data in type_dict.items():
            if isinstance(type_data, dict):
                fields.update(_extract_typed_fields(type_key, type_data))

    fields.update(_extract_extra_fields(content_wrapper.get("extra_fields", [])))
    for section in content_wrapper.get("extra_sections", []):
        fields.update(_extract_extra_fields(section.get("extra_fields", [])))

    if not fields:
        raise AnsibleLookupError(
            f"proton_pass: no fields extracted from item '{title}'. "
            "Ensure the item has a note, standard fields, or at least one extra "
            "hidden field. Set the lookup option debug=true to inspect the raw JSON."
        )

    return fields


def _extract_extra_fields(extra_fields: list[dict]) -> dict[str, str]:
    """Return a flat ``{field_name: value}`` dict from a pass-cli extra_fields list.

    Each entry has the shape::

        {"name": "key", "content": {"Hidden"|"Text"|"Totp"|"Timestamp": value}}

    The type wrapper key varies; the actual value is always the first dict value.
    Non-string values (for example ``Timestamp`` integers) are converted to strings.
    """
    result: dict[str, str] = {}
    for extra in extra_fields:
        name = extra.get("name", "")
        if not name:
            continue
        content = extra.get("content")
        if isinstance(content, dict):
            raw_value = next((v for v in content.values() if v is not None), "")
            result[name] = str(raw_value)
    return result


# ---------------------------------------------------------------------------
# Lookup module
# ---------------------------------------------------------------------------


class LookupModule(LookupBase):
    def run(self, terms: list[str], variables: dict | None = None, **kwargs) -> list:
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)
        vault = self.get_option("vault") or ""
        pat = self.get_option("pat") or ""
        agent_reason = self.get_option("agent_reason") or ""
        cli_path = self.get_option("cli_path") or "pass-cli"
        timeout = int(self.get_option("timeout"))
        debug = boolean(self.get_option("debug"))
        field_opt = self.get_option("field") or ""

        if not terms:
            raise AnsibleLookupError(
                "community.general.proton_pass requires at least one positional argument (item title)"
            )

        # Two-term shorthand: lookup('proton_pass', 'item_title', 'field_name')
        if len(terms) == 2:
            item_titles = [terms[0]]
            field = field_opt or terms[1]
        else:
            item_titles = list(terms)
            field = field_opt

        if len(item_titles) > 1 and not field:
            raise AnsibleOptionsError(
                "community.general.proton_pass: fetching all fields for multiple "
                "items in one call is not supported. Specify 'field' or look up "
                "one item at a time."
            )

        client = ProtonPassClient(cli_path=cli_path, timeout=timeout, agent_reason=agent_reason, debug=debug)
        client.ensure_authenticated(pat)

        results: list[str | dict[str, str | list[str]]] = []
        for title in item_titles:
            if field:
                results.append(client.fetch_field(vault=vault, title=title, field=field))
            else:
                results.append(client.fetch_all_fields(vault=vault, title=title))

        return results



#!/usr/bin/python
#
# Copyright (c) 2015, Steve Gargan <steve.gargan@gmail.com>
# Copyright (c) 2018 Genome Research Ltd.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: consul_kv
short_description: Manipulate entries in the key/value store of a Consul cluster
description:
  - Allows the retrieval, addition, modification and deletion of key/value entries in a Consul cluster using the agent. The
    entire contents of the record, including the indices, flags and session are returned as RV(ignore:value).
  - If the O(key) represents a prefix then note that when a value is removed, the existing value if any is returned as part
    of the results.
  - See http://www.consul.io/docs/agent/http.html#kv for more details.
author:
  - Steve Gargan (@sgargan)
  - Colin Nolan (@colin-nolan)
extends_documentation_fragment:
  - community.general._consul
  - community.general._consul.actiongroup_consul
  - community.general._consul.token
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  action_group:
    version_added: 13.2.0
options:
  state:
    description:
      - The action to take with the supplied key and value. If the state is V(present) and O(value) is set, the key contents
        is set to the value supplied and RV(ignore:changed) is set to V(true) only if the value was different to the current contents.
        If the state is V(present) and O(value) is not set, the existing value associated to the key is returned. The state
        V(absent) is used to remove the key/value pair, again RV(ignore:changed) is set to V(true) only if the key actually existed
        prior to the removal. An attempt can be made to obtain or free the lock associated with a key/value pair with the
        states V(acquire) or V(release) respectively. A valid session must be supplied to make the attempt RV(ignore:changed) is V(true)
        if the attempt is successful, V(false) otherwise.
    type: str
    choices: [absent, acquire, present, release]
    default: present
  key:
    description:
      - The key at which the value should be stored.
    type: str
    required: true
  value:
    description:
      - The value should be associated with the given key, required if O(state) is V(present).
    type: str
  recurse:
    description:
      - If the key represents a prefix, each entry with the prefix can be retrieved by setting this to V(true).
    type: bool
  retrieve:
    description:
      - If the O(state) is V(present) and O(value) is set, perform a read after setting the value and return this value.
    default: true
    type: bool
  session:
    description:
      - The session that should be used to acquire or release a lock associated with a key/value pair.
    type: str
  cas:
    description:
      - Used when acquiring a lock with a session. If the O(cas) is V(0), then Consul only puts the key if it does not already
        exist. If the O(cas) value is non-zero, then the key is only set if the index matches the ModifyIndex of that key.
    type: str
  flags:
    description:
      - Opaque positive integer value that can be passed when setting a value.
    type: str
  datacenter:
    description:
      - The name of the datacenter to query. If unspecified, the query defaults to the datacenter of the Consul agent on O(host).
    type: str
    version_added: 10.0.0
"""


EXAMPLES = r"""
# If the key does not exist, the value associated to the "data" property in `retrieved_key` will be `None`
# If the key value is empty string, `retrieved_key["data"]["Value"]` will be `None`
- name: Retrieve a value from the key/value store
  community.general.consul_kv:
    key: somekey
  register: retrieved_key

- name: Add or update the value associated with a key in the key/value store
  community.general.consul_kv:
    key: somekey
    value: somevalue

- name: Remove a key from the store
  community.general.consul_kv:
    key: somekey
    state: absent

- name: Add a node to an arbitrary group using Consul inventory (see consul.ini)
  community.general.consul_kv:
    key: ansible/groups/dc1/somenode
    value: top_secret

- name: Register a key/value pair with an associated session
  community.general.consul_kv:
    key: stg/node/server_birthday
    value: 20160509
    session: "{{ sessionid }}"
    state: acquire
"""

import base64
from http import HTTPStatus
from urllib.parse import quote

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.general.plugins.module_utils._consul import (
    AUTH_ARGUMENTS_SPEC,
    RequestError,
    _ConsulModule,
)


def _decode_value(entry):
    if entry.get("Value") is not None:
        entry["Value"] = base64.b64decode(entry["Value"])


def _quote_key(key):
    # KV keys may contain characters that are not valid in a URL path
    # (spaces, non-ASCII, ...). Slashes are key separators and must stay raw.
    return quote(key, safe="/")


def _kv_get(consul_module, key, recurse=False, dc=None):
    params = {"dc": dc}
    if recurse:
        params["recurse"] = "true"
    try:
        body, headers = consul_module.request("GET", ("kv", _quote_key(key)), params=params)
    except RequestError as e:
        if e.status == HTTPStatus.NOT_FOUND:
            index = e.response_headers.get("X-Consul-Index") if e.response_headers else None
            return index, None
        raise
    index = headers.get("X-Consul-Index") if headers is not None else None
    if not body:
        return index, None
    for entry in body:
        _decode_value(entry)
    if recurse:
        return index, body
    return index, body[0]


def _kv_put(consul_module, key, value, cas=None, acquire=None, release=None, flags=None, dc=None):
    params = {"cas": cas, "acquire": acquire, "release": release, "flags": flags, "dc": dc}
    if value is not None:
        value = value.encode("utf-8")
    return consul_module.put(("kv", _quote_key(key)), data=value, params=params) is True


def _kv_delete(consul_module, key, recurse=False, dc=None):
    params = {"dc": dc}
    if recurse:
        params["recurse"] = "true"
    return consul_module.delete(("kv", _quote_key(key)), params=params) is True


def _has_value_changed(consul_module, key, target_value):
    index, existing = _kv_get(consul_module, key)
    if not existing:
        return index, True
    try:
        changed = to_text(existing["Value"], errors="surrogate_or_strict") != target_value
        return index, changed
    except UnicodeError:
        # Existing value was not decodable but all values we set are valid utf-8
        return index, True


def execute(module, consul_module):
    state = module.params["state"]

    if state == "acquire" or state == "release":
        lock(module, consul_module, state)
    elif state == "present":
        if module.params["value"] is None:
            get_value(module, consul_module)
        else:
            set_value(module, consul_module)
    elif state == "absent":
        remove_value(module, consul_module)
    else:
        module.exit_json(msg=f"Unsupported state: {state}")


def lock(module, consul_module, state):
    session = module.params["session"]
    key = module.params["key"]
    value = module.params["value"]
    dc = module.params["datacenter"]

    if not session:
        module.fail_json(msg=f"{state} of lock for {key} requested but no session supplied")

    index, changed = _has_value_changed(consul_module, key, value)

    if changed and not module.check_mode:
        kwargs = {
            "cas": module.params["cas"],
            "flags": module.params["flags"],
            "dc": dc,
        }
        if state == "acquire":
            kwargs["acquire"] = session
        else:
            kwargs["release"] = session
        changed = _kv_put(consul_module, key, value, **kwargs)

    module.exit_json(changed=changed, index=index, key=key)


def get_value(module, consul_module):
    key = module.params["key"]
    index, existing = _kv_get(
        consul_module,
        key,
        recurse=module.params["recurse"],
        dc=module.params["datacenter"],
    )
    module.exit_json(changed=False, index=index, data=existing)


def set_value(module, consul_module):
    key = module.params["key"]
    value = module.params["value"]
    dc = module.params["datacenter"]

    if value is None:
        raise AssertionError(f'Cannot set value of "{key}" to None')

    index, changed = _has_value_changed(consul_module, key, value)

    if changed and not module.check_mode:
        changed = _kv_put(
            consul_module,
            key,
            value,
            cas=module.params["cas"],
            flags=module.params["flags"],
            dc=dc,
        )

    stored = None
    if module.params["retrieve"]:
        index, stored = _kv_get(consul_module, key, dc=dc)

    module.exit_json(changed=changed, index=index, key=key, data=stored)


def remove_value(module, consul_module):
    """Remove the value associated with the given key. If the recurse parameter
    is set then any key prefixed with the given key will be removed."""
    key = module.params["key"]
    dc = module.params["datacenter"]
    recurse = module.params["recurse"]

    index, existing = _kv_get(consul_module, key, recurse=recurse, dc=dc)

    changed = existing is not None
    if changed and not module.check_mode:
        _kv_delete(consul_module, key, recurse=recurse, dc=dc)

    module.exit_json(changed=changed, index=index, key=key, data=existing)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            cas=dict(type="str"),
            datacenter=dict(type="str"),
            flags=dict(type="str"),
            key=dict(type="str", required=True, no_log=False),
            recurse=dict(type="bool"),
            retrieve=dict(type="bool", default=True),
            state=dict(type="str", default="present", choices=["absent", "acquire", "present", "release"]),
            value=dict(type="str"),
            session=dict(type="str"),
            **AUTH_ARGUMENTS_SPEC,
        ),
        supports_check_mode=True,
    )
    consul_module = _ConsulModule(module)

    try:
        execute(module, consul_module)
    except RequestError as e:
        body = e.response_data
        if isinstance(body, bytes):
            body = body.decode("utf-8", errors="replace")
        body = (body or "").strip()
        module.fail_json(msg=body or f"HTTP {e.status}")
    except Exception as e:
        module.fail_json(msg=str(e))


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Yuwei Zhou, <yuwzho@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_iotdevice_info
version_added: "0.1.2"
short_description: Facts of Azure IoT hub device
description:
    - Query, get Azure IoT hub device.
options:
    hub:
        description:
            - Name of IoT Hub.
        type: str
        required: true
    hub_policy_name:
        description:
            - Policy name of the IoT Hub which will be used to query from IoT hub.
            - This policy should have at least 'Registry Read' access.
        type: str
        required: true
    hub_policy_key:
        description:
            - Key of the I(hub_policy_name).
        type: str
        required: true
    name:
        description:
            - Name of the IoT hub device identity.
        type: str
        aliases:
            - device_id
    module_id:
        description:
            - Name of the IoT hub device module.
            - Must use with I(device_id) defined.
        type: str
    query:
        description:
            - Query an IoT hub to retrieve information regarding device twins using a SQL-like language.
            - "See U(https://docs.microsoft.com/en-us/azure/iot-hub/iot-hub-devguide-query-language)."
        type: str
    top:
        description:
            - Used when I(name) not defined.
            - List the top n devices in the query.
        type: int
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Yuwei Zhou (@yuwzho)
'''

EXAMPLES = '''
- name: Get the details of a device
  azure_rm_iotdevice_info:
      name: Testing
      hub: MyIoTHub
      hub_policy_name: registryRead
      hub_policy_key: XXXXXXXXXXXXXXXXXXXX

- name: Query devices
  azure_rm_iotdevice_info:
      hub: "hub{{ rpfx }}"
      query: "SELECT * FROM devices"
      hub_policy_name: "{{ registry_write_name }}"
      hub_policy_key: "{{ registry_write_key }}"

- name: Query all device modules in an IoT Hub
  azure_rm_iotdevice_info:
      hub: MyIoTHub
      hub_policy_name: registryRead
      hub_policy_key: XXXXXXXXXXXXXXXXXXXX

- name: List all devices in an IoT Hub
  azure_rm_iotdevice_info:
      hub: MyIoTHub
      hub_policy_name: registryRead
      hub_policy_key: XXXXXXXXXXXXXXXXXXXX
'''

RETURN = '''
iot_devices:
    description:
       - IoT Hub device.
    returned: always
    type: dict
    sample: {
        "authentication": {
            "symmetricKey": {
                "primaryKey": "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
                "secondaryKey": "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"
            },
            "type": "sas",
            "x509Thumbprint": {
                "primaryThumbprint": null,
                "secondaryThumbprint": null
            }
        },
        "capabilities": {
            "iotEdge": false
        },
        "changed": true,
        "cloudToDeviceMessageCount": 0,
        "connectionState": "Disconnected",
        "connectionStateUpdatedTime": "0001-01-01T00:00:00",
        "deviceId": "Testing",
        "etag": "NzA2NjU2ODc=",
        "failed": false,
        "generationId": "636903014505613307",
        "lastActivityTime": "0001-01-01T00:00:00",
        "modules": [
            {
                "authentication": {
                    "symmetricKey": {
                        "primaryKey": "XXXXXXXXXXXXXXXXXXX",
                        "secondaryKey": "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"
                    },
                    "type": "sas",
                    "x509Thumbprint": {
                        "primaryThumbprint": null,
                        "secondaryThumbprint": null
                    }
                },
                "cloudToDeviceMessageCount": 0,
                "connectionState": "Disconnected",
                "connectionStateUpdatedTime": "0001-01-01T00:00:00",
                "deviceId": "testdevice",
                "etag": "MjgxOTE5ODE4",
                "generationId": "636903840872788074",
                "lastActivityTime": "0001-01-01T00:00:00",
                "managedBy": null,
                "moduleId": "test"
            }
        ],
        "properties": {
            "desired": {
                "$metadata": {
                    "$lastUpdated": "2019-04-10T05:00:46.2702079Z",
                    "$lastUpdatedVersion": 8,
                    "period": {
                        "$lastUpdated": "2019-04-10T05:00:46.2702079Z",
                        "$lastUpdatedVersion": 8
                    }
                },
                "$version": 1,
                "period": 100
            },
            "reported": {
                "$metadata": {
                    "$lastUpdated": "2019-04-08T06:24:10.5613307Z"
                },
                "$version": 1
            }
        },
        "status": "enabled",
        "statusReason": null,
        "statusUpdatedTime": "0001-01-01T00:00:00",
        "tags": {
            "location": {
                "country": "us",
                "city": "Redmond"
            },
            "sensor": "humidity"
        }
    }
'''  # NOQA

import traceback

from ansible.module_utils.basic import missing_required_lib
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.iot.hub import IoTHubRegistryManager
except ImportError:
    AZURE_IOT_HUB_IMPORT_ERROR = traceback.format_exc()
else:
    AZURE_IOT_HUB_IMPORT_ERROR = None


class AzureRMIoTDeviceFacts(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str', aliases=['device_id']),
            module_id=dict(type='str'),
            query=dict(type='str'),
            hub=dict(type='str', required=True),
            hub_policy_name=dict(type='str', required=True),
            hub_policy_key=dict(type='str', no_log=True, required=True),
            top=dict(type='int')
        )

        self.results = dict(
            changed=False,
            iot_devices=[]
        )

        self.name = None
        self.module_id = None
        self.hub = None
        self.hub_policy_name = None
        self.hub_policy_key = None
        self.top = None
        self.query = None

        self.mgmt_client = None
        self._base_url = None

        super(AzureRMIoTDeviceFacts, self).__init__(self.module_arg_spec, supports_check_mode=True)

        if AZURE_IOT_HUB_IMPORT_ERROR:
            self.fail(msg=missing_required_lib('azure-iot-hub', reason=("This module requires the Azure IoT Hub data-plane SDK (azure-iot-hub). "
                                                                        "Installation may fail on some platforms due to uamqp wheel build issues.")),
                      exception=AZURE_IOT_HUB_IMPORT_ERROR)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec.keys():
            setattr(self, key, kwargs[key])

        self._base_url = '{0}.azure-devices.net'.format(self.hub)

        connect_str = "HostName={0};SharedAccessKeyName={1};SharedAccessKey={2}".format(self._base_url, self.hub_policy_name, self.hub_policy_key)
        self.mgmt_client = IoTHubRegistryManager.from_connection_string(connect_str)

        response = []
        if self.module_id:
            response = [self.get_device_module()]
        elif self.name:
            response = [self.get_device()]
        elif self.query is not None:
            response = self.hub_query()
        else:
            response = self.list_devices()

        self.results['iot_devices'] = response
        return self.results

    def get_device(self):
        try:
            response = self.mgmt_client.get_device(self.name)

            response = self.format_item(response)
            return response
        except Exception as exc:
            self.fail('Error when getting IoT Hub device {0}: {1}'.format(self.name, exc))

    def get_device_module(self):
        try:
            response = self.mgmt_client.get_module(self.name, self.module_id)
            return self.format_module(response)
        except Exception as exc:
            self.fail('Error when getting IoT Hub device {0}: {1}'.format(self.name, exc))

    def list_device_modules(self):
        try:
            response = self.mgmt_client.get_modules(self.name)

            return [self.format_module(item) for item in response]
        except Exception as exc:
            self.fail('Error when getting IoT Hub device {0}: {1}'.format(self.name, exc))

    def list_devices(self):
        try:
            response = None
            response = self.mgmt_client.get_devices(max_number_of_devices=1000)

            response = [self.format_item(item) for item in response]
            if self.top is not None:
                return response[self.top - 1]
            else:
                return response
        except Exception as exc:
            if hasattr(exc, 'message'):
                pass
            else:
                self.fail('Error when listing IoT Hub devices in {0}: {1}'.format(self.hub, exc))

    def hub_query(self):
        try:
            response = None
            response = self.mgmt_client.query_iot_hub(dict(query=self.query))

            return [self.format_twin(item) for item in response.items]

        except Exception as exc:
            if hasattr(exc, 'message'):
                pass
            else:
                self.fail('Error when listing IoT Hub devices in {0}: {1}'.format(self.hub, exc))

    def format_module(self, item):
        if not item:
            return None
        format_item = dict(
            authentication=dict(),
            cloudToDeviceMessageCount=item.cloud_to_device_message_count,
            connectionState=item.connection_state,
            connectionStateUpdatedTime=item.connection_state_updated_time,
            deviceId=item.device_id,
            etag=item.etag,
            generationId=item.generation_id,
            lastActivityTime=item.last_activity_time,
            managedBy=item.managed_by,
            moduleId=item.module_id
        )
        if item.authentication:
            format_item['authentication']['symmetricKey'] = dict()
            format_item['authentication']['symmetricKey']['primaryKey'] = item.authentication.symmetric_key.primary_key
            format_item['authentication']['symmetricKey']['secondaryKey'] = item.authentication.symmetric_key.secondary_key

            format_item['authentication']['type'] = item.authentication.type
            format_item['authentication']["x509Thumbprint"] = dict()
            format_item['authentication']["x509Thumbprint"]["primaryThumbprint"] = item.authentication.x509_thumbprint.primary_thumbprint
            format_item['authentication']["x509Thumbprint"]['secondaryThumbprint'] = item.authentication.x509_thumbprint.secondary_thumbprint

        return format_item

    def format_item(self, item):
        if not item:
            return None
        format_item = dict(
            authentication=dict(),
            capabilities=dict(),
            cloudToDeviceMessageCount=item.cloud_to_device_message_count,
            connectionState=item.connection_state,
            connectionStateUpdatedTime=item.connection_state_updated_time,
            deviceId=item.device_id,
            etag=item.etag,
            generationId=item.generation_id,
            lastActivityTime=item.last_activity_time,
            status=item.status,
            statusReason=item.status_reason
        )
        if hasattr(item, 'status_updated_time'):
            format_item['statusUpdatedTime'] = item.status_updated_time
        if hasattr(item, 'modules'):
            format_item['modules'] = item.modules
        if item.authentication:
            format_item['authentication']['symmetricKey'] = dict()
            format_item['authentication']['symmetricKey']['primaryKey'] = item.authentication.symmetric_key.primary_key
            format_item['authentication']['symmetricKey']['secondaryKey'] = item.authentication.symmetric_key.secondary_key

            format_item['authentication']['type'] = item.authentication.type
            format_item['authentication']["x509Thumbprint"] = dict()
            format_item['authentication']["x509Thumbprint"]["primaryThumbprint"] = item.authentication.x509_thumbprint.primary_thumbprint
            format_item['authentication']["x509Thumbprint"]['secondaryThumbprint'] = item.authentication.x509_thumbprint.secondary_thumbprint
        if item.capabilities:
            format_item['capabilities']["iotEdge"] = item.capabilities.iot_edge

        return format_item

    def format_twin(self, item):
        if not item:
            return None
        format_twin = dict(
            device_id=item.device_id,
            module_id=item.module_id,
            tags=item.tags,
            properties=dict(),
            etag=item.etag,
            version=item.version,
            device_etag=item.device_etag,
            status=item.status,
            cloud_to_device_message_count=item.cloud_to_device_message_count,
            authentication_type=item.authentication_type,
        )
        if item.properties is not None:
            format_twin['properties']['desired'] = item.properties.desired
            format_twin['properties']['reported'] = item.properties.reported

        return format_twin


def main():
    AzureRMIoTDeviceFacts()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_postgresqlflexiblefirewallrule_info
version_added: "2.2.0"
short_description: Get Azure PostgreSQL Flexible Firewall Rule facts
description:
    - Get facts of Azure PostgreSQL Flexible Firewall Rule.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    server_name:
        description:
            - The name of the server.
        required: True
        type: str
    name:
        description:
            - The name of the server firewall rule.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Get instance of PostgreSQL Flexible Firewall Rule
  azure_rm_postgresqlflexiblefirewallrule_info:
    resource_group: myResourceGroup
    server_name: server_name
    name: firewall_rule_name

- name: List instances of PostgreSQL Flexible Firewall Rule
  azure_rm_postgresqlflexiblefirewallrule_info:
    resource_group: myResourceGroup
    server_name: server_name
'''

RETURN = '''
rules:
    description:
        - A list of dictionaries containing facts for PostgreSQL Flexible Firewall Rule.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.DBforPostgreSQL/flexibleServers/flexibled9b/firewallRules/firewalld9b"
        server_name:
            description:
                - The name of the server.
            returned: always
            type: str
            sample: testserver
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: rule1
        start_ip_address:
            description:
                - The start IP address of the PostgreSQL firewall rule.
            returned: always
            type: str
            sample: 10.0.0.16
        end_ip_address:
            description:
                - The end IP address of the PostgreSQL firewall rule.
            returned: always
            type: str
            sample: 10.0.0.18
'''

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMPostgreSQLFlexibleFirewallRulesInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.resource_group = None
        self.server_name = None
        self.name = None
        super(AzureRMPostgreSQLFlexibleFirewallRulesInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results['firewall_rules'] = self.get()
        else:
            self.results['firewall_rules'] = self.list_by_server()
        return self.results

    def get(self):
        response = None
        try:
            response = self.postgresql_flexible_client.firewall_rules.get(resource_group_name=self.resource_group,
                                                                          server_name=self.server_name,
                                                                          firewall_rule_name=self.name)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError as e:
            self.log('Could not get facts for FirewallRules. Exception as {0}'.format(str(e)))
            return []

        return [self.format_item(response)]

    def list_by_server(self):
        response = None
        results = []
        try:
            response = self.postgresql_flexible_client.firewall_rules.list_by_server(resource_group_name=self.resource_group,
                                                                                     server_name=self.server_name)
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log('Could not get facts for FirewallRules. Exception as {0}'.format(str(e)))
            return []

        if response is not None:
            for item in response:
                results.append(self.format_item(item))

        return results

    def format_item(self, item):
        d = item.as_dict()
        d = {
            'resource_group': self.resource_group,
            'id': d['id'],
            'server_name': self.server_name,
            'name': d['name'],
            'start_ip_address': d['start_ip_address'],
            'end_ip_address': d['end_ip_address']
        }
        return d


def main():
    AzureRMPostgreSQLFlexibleFirewallRulesInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Liu Qingyi, (@smile37773)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_azurefirewall_info
version_added: '0.1.2'
short_description: Get AzureFirewall info
description:
    - Get info of AzureFirewall.
options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
    name:
        description:
            - Resource name.
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - Liu Qingyi (@smile37773)

'''

EXAMPLES = '''
- name: List all Azure Firewalls for a given subscription
  azure_rm_azurefirewall_info:
- name: List all Azure Firewalls for a given resource group
  azure_rm_azurefirewall_info:
    resource_group: myResourceGroup
- name: Get Azure Firewall
  azure_rm_azurefirewall_info:
    resource_group: myResourceGroup
    name: myAzureFirewall
'''

RETURN = '''
firewalls:
    description:
        - A list of dict results where the key is the name of the AzureFirewall and the values are the facts for that AzureFirewall.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/
                     myResourceGroup/providers/Microsoft.Network/azureFirewalls/myAzureFirewall"
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: "myAzureFirewall"
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: "eastus"
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { "tag": "value" }
        etag:
            description:
                - Gets a unique read-only string that changes whenever the resource is updated.
            returned: always
            type: str
            sample: "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
        application_rule_collections:
            description:
                - Collection of application rule collections used by Azure Firewall.
            type: list
            returned: always
            sample: [
                {
                    "etag": "25b7fca4-c909-4913-8799-6cdb7da2e298",
                    "id": "/subscriptions/xxx-xxx1/resourceGroups/myResourceGroup/providers/
                           Microsoft.Network/azureFirewalls/myFirewall/applicationRuleCollections/apprulecoll",
                    "name": "apprulecoll",
                    "properties": {
                        "action": {
                            "type": "Deny"
                        },
                        "priority": 110,
                        "provisioningState": "Failed",
                        "rules": [
                            {
                                "actions": [],
                                "description": "Deny inbound rule",
                                "direction": "Inbound",
                                "fqdnTags": [],
                                "name": "rule1",
                                "priority": 0,
                                "protocols": [
                                    {
                                        "port": 443,
                                        "protocolType": "Https"
                                    }
                                ],
                                "sourceAddresses": [
                                    "216.58.216.164",
                                    "10.0.0.0/25"
                                ],
                                "sourceIpGroups": [],
                                "targetFqdns": [
                                    "www.test.com"
                                ]
                            }
                        ]
                    },
                    "type": "Microsoft.Network/azureFirewalls/applicationRuleCollections"
                }
            ]
        nat_rule_collections:
            description:
                - Collection of NAT rule collections used by Azure Firewall.
            type: list
            returned: always
            sample: [
                {
                    "etag": '3759e8b1-cdf7-44f4-a643-3a75e2b67877',
                    "id": "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/
                           azureFirewalls/myFirewall/natRuleCollections/natrulecoll",
                    "name": "natrulecoll",
                    "properties": {
                        "action": {
                            "type": "Dnat"
                        },
                        "priority": 112,
                        "provisioningState": "Failed",
                        "rules": [
                            {
                                "description": "D-NAT all outbound web traffic for inspection",
                                "destinationAddresses": [
                                    "20.169.156.124"
                                ],
                                "destinationPorts": [
                                    "443"
                                ],
                                "name": "DNAT-HTTPS-traffic",
                                "protocols": [
                                    "TCP"
                                ],
                                "sourceAddresses": [
                                    "*"
                                ],
                                "translatedAddress": "1.2.3.5",
                                "translatedPort": "8443"
                            }
                        ]
                    },
                    "type": "Microsoft.Network/azureFirewalls/natRuleCollections"
                }
            ]
        network_rule_collections:
            description:
                - Collection of network rule collections used by Azure Firewall.
            type: list
            returned: always
            sample: [
                {
                    "etag": "3759e8b1-cdf7-44f4-a643-3a75e2b67877",
                    "id": "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/
                           azureFirewalls/myFirewall/networkRuleCollections/netrulecoll",
                    "name": "netrulecoll",
                    "properties": {
                        "action": {
                            "type": "Deny"
                        },
                        "priority": 112,
                        "provisioningState": "Failed",
                        "rules": [
                            {
                                "description": "Block traffic based on source IPs and ports",
                                "destinationAddresses": [
                                    "*"
                                ],
                                "destinationPorts": [
                                    "443-444",
                                    "8443"
                                ],
                                "name": "L4-traffic",
                                "protocols": [
                                    "TCP"
                                ],
                                "sourceAddresses": [
                                    "192.168.1.1-192.168.1.12",
                                    "10.1.4.12-10.1.4.255"
                                ]
                            },
                            {
                                "description": "Block traffic based on source IPs and ports to amazon",
                                "destinationAddresses": [],
                                "destinationPorts": [
                                    "443-444",
                                    "8443"
                                ],
                                "name": "L4-traffic-with_FQDN",
                                "protocols": [
                                    "TCP"
                                ],
                                "sourceAddresses": [
                                    "10.2.4.12-10.2.4.255"
                                ]
                            }
                        ]
                    },
                    "type": "Microsoft.Network/azureFirewalls/networkRuleCollections"
                }
            ]
        ip_configurations:
            description:
                - IP configuration of the Azure Firewall resource.
            type: list
            returned: always
            sample: [
                {
                    "etag": "3759e8b1-cdf7-44f4-a643-3a75e2b67877",
                    "id": "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/azureFirewalls/myFirewall/i
                           azureFirewallIpConfigurations/azureFirewallIpConfiguration",
                    "name": "azureFirewallIpConfiguration",
                    "properties": {
                        "privateIPAllocationMethod": "Dynamic",
                        "provisioningState": "Succeeded",
                        "publicIPAddress": {
                            "id": "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/publicIPAddresses/myPublicIpAddress"
                        },
                        "subnet": {
                            "id": "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/
                                   virtualNetworks/myVirtualNetwork/subnets/AzureFirewallSubnet"
                        }
                    },
                    "type": "Microsoft.Network/azureFirewalls/azureFirewallIpConfigurations"
                }
            ]
        provisioning_state:
            description:
                - The current state of the gallery.
            type: str
            sample: "Succeeded"

'''

import json
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient


class AzureRMAzureFirewallsInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str'
            ),
            name=dict(
                type='str'
            )
        )

        self.resource_group = None
        self.name = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200]

        self.query_parameters = {}
        self.query_parameters['api-version'] = '2024-01-01'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        self.mgmt_client = None
        super(AzureRMAzureFirewallsInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        if (self.resource_group is not None and self.name is not None):
            self.results['firewalls'] = self.get()
        elif (self.resource_group is not None):
            self.results['firewalls'] = self.list()
        else:
            self.results['firewalls'] = self.listall()
        return self.results

    def get(self):
        response = None
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.Network' +
                    '/azureFirewalls' +
                    '/{{ azure_firewall_name }}')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)
        self.url = self.url.replace('{{ azure_firewall_name }}', self.name)

        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
            # self.log('Response : {0}'.format(response))
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return self.format_item(results)

    def list(self):
        response = None
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.Network' +
                    '/azureFirewalls')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)

        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
            # self.log('Response : {0}'.format(response))
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return [self.format_item(x) for x in results['value']] if results['value'] else []

    def listall(self):
        response = None
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/providers' +
                    '/Microsoft.Network' +
                    '/azureFirewalls')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)

        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
            # self.log('Response : {0}'.format(response))
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return [self.format_item(x) for x in results['value']] if results['value'] else []

    def format_item(self, item):
        if item is None or item == {}:
            return {}
        d = {
            'id': item.get('id'),
            'name': item.get('name'),
            'location': item.get('location'),
            'etag': item.get('etag'),
            'tags': item.get('tags'),
            'nat_rule_collections': dict(),
            'network_rule_collections': dict(),
            'ip_configurations': dict(),
        }
        if isinstance(item.get('properties'), dict):
            d['nat_rule_collections'] = item.get('properties').get('natRuleCollections')
            d['network_rule_collections'] = item.get('properties').get('networkRuleCollections')
            d['application_rule_collections'] = item.get('properties').get('applicationRuleCollections')
            d['ip_configurations'] = item.get('properties').get('ipConfigurations')
            d['provisioning_state'] = item.get('properties').get('provisioningState')
        return d


def main():
    AzureRMAzureFirewallsInfo()


if __name__ == '__main__':
    main()
